--===========================================================
-- unique_materials_by_name.ms
-- Contact: martin.ashton@autodesk.com
--
-- Usage: Scripting > MaxScript Editor > File > Open > 
--            > unique_materials_by_name.ms > CTRL+E to evaluate.
--
-- If a scene contains many materials with the same name, these 
-- materials will be replaced with a shared unique instance of
-- that material.
--
-- Any unassigned materials after this script is executed will be 
-- discarded when the scene is saved.
--===========================================================
(
	-- Initialize a list of all the unique materials by name.
	uniqueMaterials = #()
	
	fn GetUniqueMaterial inputMat = 
	(
		local retval
		
		-- Collect all the materials which match this material's name.
		-- There should only be at most 1 materials in this returned array.
		materials = for m in uniqueMaterials where m.name == inputMat.name collect m
		
		if materials.count == 0 then
		(
			-- Add the material if it doesn't exist.
			append uniqueMaterials inputMat
			retval = inputMat
			
		) else (
			-- Otherwise return the first material.
			retval = materials[1]
		)
		
		retval
	)
	
	allObjects = objects as array
	for obj in allObjects do
	(
		mat = obj.material
		if mat != undefined then
		(
			unique_mat = GetUniqueMaterial mat
			obj.material = unique_mat
		)
	)
	
	ok
)
